/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#include <stdio.h>
#include <string.h>
#include <cadef.h>
#include <db_access.h>
#include <tcl.h>
#include <tk.h>
#if defined(USE_BLT)
#include <blt.h>
#endif
#include <shareLib.h>

/* Default Channel Access timeout, and magic pend event time for a poll */
#define CA_PEND_IO_TIME 10.0
#define MAGIC_POLL .000000001

typedef union {
  struct dbr_time_string s;
  struct dbr_time_enum   e;
  struct dbr_time_char   c;
  struct dbr_time_short  i;
  struct dbr_time_long   l;
  struct dbr_time_float  f;
  struct dbr_time_double d;
} DATABUF;

typedef union {
  struct dbr_sts_string  s;
  struct dbr_ctrl_enum   e;
  struct dbr_ctrl_char   c;
  struct dbr_ctrl_short  i;
  struct dbr_ctrl_long   l;
  struct dbr_ctrl_float  f;
  struct dbr_ctrl_double d;
} INFOBUF;

typedef union {
  long tclLong;
  double tclDouble;
  char *tclStr;
} TCLDATABUF;

typedef enum {NewConnection,NeverConnected, LostConnection,
	      ReadAccess,WriteAccess, PendIO, ConnectionOK } 
caState;

typedef struct MonScript {
  Tcl_DString script;
  int  number;
} MONSCRIPT;

typedef struct ViewPort {
  int    base;
  int    size;
  int    precision;
} VIEWPORT;
	
typedef struct Item {
  struct Chandata *pchandata;
  char *tclname;
#if defined(USE_BLT)
  char *vecname;
#endif
} ITEM;
	
typedef struct Chandata{
  chid 		chid;
  evid 		evid;
  DATABUF 	*pdata;
  INFOBUF  	info;
  VIEWPORT	view;
  caState  	castate;
  int	 	countPendIO;
  int     	initSize; 
  int     	dataSize;
  int	 	iType;       /* native ca type of pv               */
  int           tclType;     /* closest tcl type to native ca type */
  int	 	precision; 
  TCLDATABUF    tclValue;
  ITEM		*pitem;
  MONSCRIPT	*pscript;
  int           secondMonEvent;
  int		updateVar;
  Tcl_Interp	*interp;
  int           skip;
#if defined(USE_BLT)
  double	*pvec;
  Blt_Vector	*vecPtr;
#endif
} CHANDATA;

/***********************/
/* Function Prototypes */
/***********************/
epicsShareFunc int epicsShareAPI tclCaStat(Tcl_Interp *interp, char *tclNameList, char *arrayList);
epicsShareFunc int epicsShareAPI tclCaGet(Tcl_Interp *interp, char *tclNameList, 
	     int waitFlag, float pendioTime);
epicsShareFunc int epicsShareAPI tclCaPut(Tcl_Interp *interp, char *tclNameList, int waitFlag,
	     int quickFlag, float pendioTime);
epicsShareFunc int epicsShareAPI tclCaLink(Tcl_Interp *interp, char *tclNameList, char *pvNameList,
	      int waitFlag, float pendioTime, int blunderAhead);
epicsShareFunc int epicsShareAPI tclCaUnLink(Tcl_Interp *interp, char *tclNameList);
epicsShareFunc int epicsShareAPI tclCaMon(Tcl_Interp *interp, char *tclNameList, 
	     char *pcommand, int updateVar);
epicsShareFunc int epicsShareAPI tclCaCmon(Tcl_Interp *interp, char *tclNameList);
epicsShareFunc int epicsShareAPI tclCaVviewport(Tcl_Interp *interp, char *tclNameList, char *portParmList);
epicsShareFunc int epicsShareAPI tclCaVviewput(Tcl_Interp *interp, char *tclNameList, char *valueList);
epicsShareFunc int epicsShareAPI tclCaVviewget(Tcl_Interp *interp, char *tclNameList);
epicsShareFunc int epicsShareAPI tclCaVdis(Tcl_Interp *interp, char *tclName,
	      char *pathName, char *elemName);
epicsShareFunc int epicsShareAPI tclCaInfostate(Tcl_Interp *interp, char *tclNameList, char *pvParmList);
#if defined(USE_BLT)
epicsShareFunc int epicsShareAPI tclCaVmapToVector(Tcl_Interp *interp, char *tclNameList, char *bltNameList);
#endif

/***********************/
/* tclCaHash functions */
/***********************/
epicsShareFunc void epicsShareAPI pv_hash_init();
epicsShareFunc ITEM * epicsShareAPI pv_search(char *s);
epicsShareFunc ITEM * epicsShareAPI pv_exists(char *s);
epicsShareFunc void epicsShareAPI pv_delete(char *s);
