/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <tclCa.h>

/* Function Prototypes */
int tclCaCmd(ClientData dummy, Tcl_Interp *interp, int argc, char *argv[]);
int TclCa_Init(Tcl_Interp *interp);

int tclCaCmd(ClientData dummy, Tcl_Interp *interp, int argc, char *argv[]) {

  int retval,temp;
  float pendVal;
  float pendIO_time = CA_PEND_IO_TIME;
  
  
  if (argc < 2) {
#if defined(CATCH)
    Tcl_SetResult(interp, "pv: wrong # of args", TCL_STATIC);
    return TCL_ERROR;
#else
    Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
    Tcl_SetErrorCode(interp, "pv: wrong # of args", (char *) NULL);
    return TCL_OK;
#endif
  }
  else if (strcmp(argv[1], "stat") == 0) {
#if defined(CATCH)
    if (argc == 4) {
        return tclCaStat(interp,argv[2],argv[3]);
      } else {
      	Tcl_SetResult(interp, "stat: wrong # of args", TCL_STATIC);
      	return TCL_ERROR;
      }
#else
    if (argc == 4) {
      if (tclCaStat(interp,argv[2],argv[3]) == 0){
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      }
    } else {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "stat: wrong # of args", (char *) NULL);
    }
    return TCL_OK;
#endif
  }
  else if (strcmp(argv[1], "get") == 0) {
#if defined(CATCH)
    if (argc == 3) {
      	return tclCaGet(interp,argv[2],False,pendIO_time);
    } else if (argc == 4) {
      if ((pendVal = atof(argv[3])) == 0) {
	Tcl_SetResult(interp,"get: timeout must be a number", TCL_STATIC);
	return TCL_ERROR;
      } else {
      	return tclCaGet(interp,argv[2],False,pendVal);
      }
    } else {
      	Tcl_SetResult(interp, "stat: wrong # of args", TCL_STATIC);
      	return TCL_ERROR;
    }
#else
    if (argc == 3) {
      if ((retval=tclCaGet(interp,argv[2],False,pendIO_time)) == 0){
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
    } else if (argc == 4) {
      if ((pendVal = atof(argv[3])) == 0) {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
	Tcl_SetErrorCode(interp, argv[3],"get: timeout must be a number", 
			 (char *) NULL);
      } else if ((retval = tclCaGet(interp,argv[2],False,pendVal)) == 0){
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else { 
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
    } else {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "get: wrong # of args", (char *) NULL);
    }
    return TCL_OK;
#endif
  }
  else if (strcmp(argv[1], "getw") == 0) {
#if defined(CATCH)
    if (argc == 3) {
      return tclCaGet(interp,argv[2],True,pendIO_time);
    } else if (argc == 4) {
      if ((pendVal = atof(argv[3])) == 0 ) {
	Tcl_SetResult(interp,"getw: timeout must be a number", TCL_STATIC);
	return TCL_ERROR;
      } else {
      	return tclCaGet(interp,argv[2],True,pendVal);
      }
    } else {
      Tcl_SetResult(interp, "getw: wrong # of args", TCL_STATIC);
      return TCL_ERROR;
    }
#else
    if (argc == 3) {
      if((retval=tclCaGet(interp,argv[2],True,pendIO_time)) == 0){
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
    } else if (argc == 4) {
      if ((pendVal = atof(argv[3])) == 0 ) {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
	Tcl_SetErrorCode(interp, argv[3],"getw: timeout must be a number", 
			 (char *) NULL);
      } else if ((retval = tclCaGet(interp,argv[2],True,pendVal)) == 0){
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
    } else {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "getw: wrong # of args", (char *) NULL);
    }
    return TCL_OK;
#endif
  }
  else if (strcmp(argv[1], "put") == 0) {
#if defined(CATCH)
    if (argc == 3) {
       return tclCaPut(interp,argv[2],False,False,pendIO_time);
    } else if (argc == 4) {
      if ((pendVal = atof(argv[3])) == 0 ) {
	Tcl_SetResult(interp,"put: timeout must be a number", TCL_STATIC);
	return TCL_ERROR;
      } else {
      	return tclCaPut(interp,argv[2],False,False,pendVal);
      }
    } else {
      Tcl_SetResult(interp, "put: wrong # of args", TCL_STATIC);
      return TCL_ERROR;
    }
#else
    if (argc == 3) {
      if ((retval=tclCaPut(interp,argv[2],False,False,pendIO_time)) == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
      return TCL_OK;
    } else if (argc == 4) {
      if ((pendVal = atof(argv[3])) == 0 ) {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
	Tcl_SetErrorCode(interp, argv[3],"put: timeout must be a number", 
			 (char *) NULL);
      } else if ((retval=tclCaPut(interp,argv[2],False,False,pendVal)) == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else { 
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
      return TCL_OK;
    } else {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "put: wrong # of args", (char *) NULL);
      return TCL_OK;
    }
#endif
  }
  else if (strcmp(argv[1], "putw") == 0) {
#if defined(CATCH)
    if (argc == 3) {
      	return tclCaPut(interp,argv[2],True,False,pendIO_time);
    } else if (argc == 4) {
      if ((pendVal = atof(argv[3])) == 0) {
	Tcl_SetResult(interp, "putw: timeout must be a number", TCL_STATIC);
	return TCL_ERROR;
      } else {
      	return tclCaPut(interp,argv[2],True,False,pendVal);
      }
    } else {
      Tcl_SetResult(interp, "putw: wrong # of args", TCL_STATIC);
      return TCL_ERROR;
    }
#else
    if (argc == 3) {
      if ((retval=tclCaPut(interp,argv[2],True,False,pendIO_time)) == 0){
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
      return TCL_OK;
    } else if (argc == 4) {
      if ((pendVal = atof(argv[3])) == 0) {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
	Tcl_SetErrorCode(interp, argv[3],"putw: timeout must be a number", 
			 (char *) NULL);
      } else if ((retval=tclCaPut(interp,argv[2],True,False,pendVal)) == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
      return TCL_OK;
    } else {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "putw: wrong # of args", (char *) NULL);
      return TCL_OK;
    }
#endif
  }
  else if (strcmp(argv[1], "putq") == 0) {
#if defined(CATCH)
    if (argc == 3) {
      	return tclCaPut(interp,argv[2],True,True,pendIO_time);
    } else if (argc == 4) {
      if ((pendVal = atof(argv[3])) == 0) {
	Tcl_SetResult(interp,"putq: timeout must be a number", TCL_STATIC);
	return TCL_ERROR;
      } else {
      	return tclCaPut(interp,argv[2],True,True,pendVal);
     }
    } else {
      Tcl_SetResult(interp, "putq: wrong # of args", TCL_STATIC);
      return TCL_ERROR;
    }
#else
    if (argc == 3) {
      if ((retval=tclCaPut(interp,argv[2],True,True,pendIO_time)) == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
      return TCL_OK;
    } else if (argc == 4) {
      if ((pendVal = atof(argv[3])) == 0) {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
	Tcl_SetErrorCode(interp, argv[3],"putq: timeout must be a number", 
			 (char *) NULL);
      } else if ((retval=tclCaPut(interp,argv[2],True,True,pendVal)) == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
      return TCL_OK;
    } else {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "putq: wrong # of args", (char *) NULL);
      return TCL_OK;
    }
#endif
  }
  else if (strcmp(argv[1], "link") == 0) {
#if defined(CATCH)
    if (argc == 4) {
      	return tclCaLink(interp,argv[2],argv[3],False,pendIO_time,0);
    } else if (argc == 5) {
      if ((pendVal = atof(argv[4])) == 0) {
	Tcl_SetResult(interp,": link timeout must be a number:", TCL_STATIC);
	return TCL_ERROR;
      } else {
      	return tclCaLink(interp,argv[2],argv[3],False,pendVal,0); 
      }
    } else {
      Tcl_SetResult(interp, "link: wrong # of args ", TCL_STATIC);
      return TCL_ERROR;
    }
#else
    if (argc == 4) {
      if ((retval=tclCaLink(interp,argv[2],argv[3],False,pendIO_time,0)) 
	  == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
    } else if (argc == 5) {
      if ((pendVal = atof(argv[4])) == 0) {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
	Tcl_SetErrorCode(interp, argv[4],": link timeout must be a number:", 
			 (char *) NULL);
      } else if ((retval=tclCaLink(interp,argv[2],argv[3],False,pendVal,0)) 
		 == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else { 
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
    } else {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "link: wrong # of args ", (char *) NULL);
    }
    return TCL_OK;
#endif
  }
  else if (strcmp(argv[1], "linkw") == 0) {
#if defined(CATCH)
    if (argc == 4) {
      	return tclCaLink(interp,argv[2],argv[3],True,pendIO_time,0);
    } else if (argc == 5) {
      if (strcmp(argv[4], "-blunderAhead") == 0) {
	return tclCaLink(interp,argv[2],argv[3],True,pendIO_time,1);
      } else if ((pendVal = atof(argv[4])) == 0) {
	Tcl_SetResult(interp,": linkw timeout must be a number", TCL_STATIC);
	return TCL_ERROR;
      } else {
      	return tclCaLink(interp,argv[2],argv[3],True,pendVal,0);
      }
    } else if (argc == 6) {
      if (((pendVal = atof(argv[4])) == 0) || (strcmp(argv[5], "-blunderAhead") != 0)) {
	Tcl_SetResult(interp,": linkw timeout must be a number and last option must be -blunderAhead", TCL_STATIC);
	return TCL_ERROR;
      }
      return tclCaLink(interp,argv[2],argv[3],True,pendVal,1);
    } else {
      Tcl_SetResult(interp, "linkw: wrong # of args", TCL_STATIC);
      return TCL_ERROR;
    }
#else
    if (argc == 4) {
      if ((retval=tclCaLink(interp,argv[2],argv[3],True,pendIO_time,0)) 
	  == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
    } else if (argc == 5) {
      if (strcmp(argv[4], "-blunderAhead") == 0) {
	if ((retval=tclCaLink(interp,argv[2],argv[3],True,pendIO_time,1)) 
	    == 0) {
	  Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
	} else {
	  Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
	}
      } else if ((pendVal = atof(argv[4])) == 0) {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
	Tcl_SetErrorCode(interp, argv[4],": linkw timeout must be a number or -blunderAhead", 
			 (char *) NULL);
      } else if ((retval=tclCaLink(interp,argv[2],argv[3],True,pendVal,0)) 
		 == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
    } else if (argc == 6) {
      if (((pendVal = atof(argv[4])) == 0) || (strcmp(argv[5], "-blunderAhead") != 0)) {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
	Tcl_SetErrorCode(interp, argv[4],": linkw timeout must be a number and last option must be -blunderAhead", 
			 (char *) NULL);
      } else if ((retval=tclCaLink(interp,argv[2],argv[3],True,pendVal,1)) 
		 == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
    } else {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "linkw: wrong # of args", (char *) NULL);
    }
    return TCL_OK;
#endif
  }
  else if ( (strcmp(argv[1], "mon") == 0) ||
            (strcmp(argv[1], "umon") == 0)) {
    temp=0;
#if defined(CATCH)
    if (strcmp(argv[1], "umon") == 0) temp =1; 
    if (argc == 3) {
      	return tclCaMon(interp,argv[2],NULL,temp);
    } else if (argc == 4) {
      	return tclCaMon(interp,argv[2],argv[3],temp);
    } else {
      Tcl_SetResult(interp, "mon: wrong # of args", TCL_STATIC);
      return TCL_ERROR;
    }
#else
    if (strcmp(argv[1], "umon") == 0) 
      temp =1; 
    if (argc == 3) {
      if (tclCaMon(interp,argv[2],NULL,temp) == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      }
    } else if (argc == 4) {
      if (tclCaMon(interp,argv[2],argv[3],temp) == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      }
    } else {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "mon: wrong # of args", (char *) NULL);
    }
    return TCL_OK;
#endif
  }
  else if (strcmp(argv[1], "cmon") == 0) {
#if defined(CATCH)
    if (argc == 3) {
      	return tclCaCmon(interp,argv[2]);
    } else {
      Tcl_SetResult(interp, "cmon: wrong # of args", TCL_STATIC);
      return TCL_ERROR;
    }
#else
    if (argc == 3) {
      if (tclCaCmon(interp,argv[2]) == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      }
    } else {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "cmon: wrong # of args", (char *) NULL);
    }
    return TCL_OK;
#endif
  }
  else if (strcmp(argv[1], "vdef") == 0) {
#if defined(CATCH)
    if (argc == 3) {
      	return tclCaVviewport(interp,argv[2],"{}");
    } else if (argc == 4) {
      	return tclCaVviewport(interp,argv[2],argv[3]);
    } else {
      Tcl_SetResult(interp, "vdef: wrong # of args", TCL_STATIC);
      return TCL_ERROR;
    }
#else
    if (argc == 3) {
      if (tclCaVviewport(interp,argv[2],"{}") == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      }
    }
    else if (argc == 4) {
      if (tclCaVviewport(interp,argv[2],argv[3]) == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      }
    } else {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "vdef: wrong # of args", (char *) NULL);
    }
    return TCL_OK;
#endif
  }
  else if (strcmp(argv[1], "vset") == 0) {
#if defined(CATCH)
    if (argc == 3) {
      	return tclCaVviewget(interp,argv[2]);
    } else if (argc == 4) {
      	return tclCaVviewput(interp,argv[2],argv[3]);
    } else {
      Tcl_SetResult(interp, "vset: wrong # of args", TCL_STATIC);
      return TCL_ERROR;
    }
#else
    if (argc == 3) {
      if (tclCaVviewget(interp,argv[2]) != 0) {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      }
    }
    else if (argc == 4) {
      if (tclCaVviewput(interp,argv[2],argv[3]) != 0) {
	Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      }
    } else {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "vset: wrong # of args", (char *) NULL);
    }
    return TCL_OK;
#endif
  }
  else if (strcmp(argv[1], "vmap") == 0) {
#if !defined(USE_BLT)
    Tcl_SetResult(interp, "vmap: not available in this version", TCL_STATIC);
    return TCL_ERROR;
#else
#if defined(CATCH)
    if (argc == 4) {
      	return tclCaVmapToVector(interp,argv[2],argv[3]); 
    } else {
      Tcl_SetResult(interp, "vmap: wrong # of args ", TCL_STATIC);
      return TCL_ERROR;
    }
#else
    if (argc == 4) {
      if ((retval=tclCaVmapToVector(interp,argv[2],argv[3])) 
	  == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
    } else {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "vmap: wrong # of args ", (char *) NULL);
    }
    return TCL_OK;
#endif
#endif
  }
  else if (strcmp(argv[1], "info") == 0) {
#if defined(CATCH)
    if (argc != 4) {
      Tcl_SetResult(interp, "info: wrong # of args", TCL_STATIC);
      return TCL_ERROR;
    }
    return tclCaInfostate(interp,argv[2],argv[3]);
#else
    if (argc != 4) {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "info: wrong # of args", (char *) NULL);
      return TCL_OK;
    }
    if (tclCaInfostate(interp,argv[2],argv[3]) == -1) {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
    } 
    return TCL_OK;
#endif
  }
  else if (strcmp(argv[1], "unlink") == 0) {
#if defined(CATCH)
    if (argc == 3) {
      	return tclCaUnLink(interp,argv[2]);
    } else {
      Tcl_SetResult(interp, "unlink: wrong # of args ", TCL_STATIC);
      return TCL_ERROR;
    }
#else
    if (argc == 3) {
      if ((retval=tclCaUnLink(interp,argv[2])) == 0) {
	Tcl_SetResult(interp, "0", (Tcl_FreeProc *) NULL);
      } else {
	Tcl_SetResult(interp, "1",(Tcl_FreeProc *) NULL);
      }
    } else {
      Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
      Tcl_SetErrorCode(interp, "unlink: wrong # of args ", (char *) NULL);
    }
    return TCL_OK;
#endif
  }
  else {
#if defined(CATCH)
    Tcl_AppendResult(interp, argv[1], "Unknown command type ", NULL);
    return TCL_ERROR;
#else
    Tcl_SetErrorCode(interp, argv[1], "Unknown command type ", (char *) NULL);
    Tcl_SetResult(interp, "1", (Tcl_FreeProc *) NULL);
    return TCL_OK;
#endif
  }
}

int TclCa_Init(Tcl_Interp *interp) {
  Tcl_CreateCommand(interp, "pv", tclCaCmd, (ClientData *) NULL, \
		    (Tcl_CmdDeleteProc *)NULL);
  pv_hash_init();
  return TCL_OK;
}
