/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#include <stdlib.h>
#define epicsExportSharedSymbols
#include "tclCa.h"

static Tcl_HashTable tclCaLinkHash;

void epicsShareAPI pv_hash_init() {
  Tcl_InitHashTable(&tclCaLinkHash, TCL_STRING_KEYS);
}

/****************************************************************
 * find 's' in the hash table, if not found allocate space for it 
****************************************************************/
ITEM * epicsShareAPI pv_search(char *s) {
  CHANDATA *pchandata;
  ITEM *try;
  int newEntry;
  Tcl_HashEntry *entryPtr;

  entryPtr = Tcl_CreateHashEntry(&tclCaLinkHash, s, &newEntry);

  if (newEntry) {
    try = (ITEM *)calloc(1,sizeof(ITEM));
    if (try == NULL)
      return NULL;
    pchandata = (CHANDATA *)calloc(1,sizeof(CHANDATA));
    if (pchandata == NULL) {
      free(try);
      return NULL;
    }
    try->tclname = (char *)malloc(strlen(s)+1);
#if defined(USE_BLT)
    try->vecname = NULL;
#endif
    pchandata->castate = NewConnection;
    try->pchandata = pchandata;
    strcpy(try->tclname,s);
    Tcl_SetHashValue(entryPtr, try);
    return try;

  } else {
    try = (ITEM *)Tcl_GetHashValue(entryPtr);
    return try;
  }
}

/****************************************************************
 * find 's' in the hash table
****************************************************************/
ITEM * epicsShareAPI pv_exists(char *s) {
  ITEM *try;
  Tcl_HashEntry *entryPtr;
  
  entryPtr = Tcl_FindHashEntry(&tclCaLinkHash, s);
  if (entryPtr == NULL) {
    return NULL;
  } else {
    try = (ITEM *)Tcl_GetHashValue(entryPtr);
    return try;
  }
}

void  epicsShareAPI pv_delete(char *s) {
  Tcl_HashEntry *entryPtr;

  entryPtr = Tcl_FindHashEntry(&tclCaLinkHash, s);
  if (entryPtr == NULL) {
    return;
  } else {
    Tcl_DeleteHashEntry(entryPtr);
    return;
  }
}
