/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/

#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include "tclSDDS.h"

static Tcl_HashTable tclSDDSCmdTable;

typedef struct {
  char *name;     /* sdds xxx command name */
  int ( epicsShareAPI *proc)();  /* command handling function */
} CMD_INFO;

static CMD_INFO tclSDDSCommands[] = {
  {"open", tclSDDSOpen},
  {"close", tclSDDSClose},
  {"getRowCount", tclSDDSGetRowCount},
  {"getDescription", tclSDDSGetDescription},
  {"getColumn", tclSDDSGetColumn},
  {"getColumnInformation", tclSDDSGetColumnInformation},
  {"getParameter", tclSDDSGetParameter},
  {"getParameterInformation", tclSDDSGetParameterInformation},
  {"getNames", tclSDDSGetNames},
  {"getArray", tclSDDSGetArray},
  {"getArrayInformation", tclSDDSGetArrayInformation}, 
  {"defineColumn", tclSDDSDefineColumn},
  {"defineParameter", tclSDDSDefineParameter},
  {"defineArray", tclSDDSDefineArray},
  {"startPage", tclSDDSStartPage},
  {"writePage", tclSDDSWritePage},
  {"writeLayout", tclSDDSWriteLayout},
  {"setDescription", tclSDDSSetDescription},
  {"setParameter", tclSDDSSetParameter},
  {"setArray", tclSDDSSetArray},
  {"setRowValues", tclSDDSSetRowValues},
  {"setColumn", tclSDDSSetColumn},
  {"check", tclSDDSCheck},
  {"collapse", tclSDDSCollapse},
  {"expand", tclSDDSExpand},
  {"convert", tclSDDSConvert},
  {"load", tclSDDSLoad},
  {"save", tclSDDSSave},
  {(char *)NULL, (int (epicsShareAPI *)())NULL}
};


/* Function Prototypes */
int tclSDDSCmd(ClientData dummy, Tcl_Interp *interp, int argc, char *argv[]);
int TclSDDS_Init(Tcl_Interp *interp);

int tclSDDSCmd(ClientData dummy, Tcl_Interp *interp, int argc, char *argv[]) {

  Tcl_HashEntry *entryPtr;
  int (*handler)();

  if (argc < 2) {
    Tcl_SetResult(interp, "sdds: must specify a sub-command", 
		  (Tcl_FreeProc *) NULL); 
    return TCL_ERROR;
  }

  entryPtr = Tcl_FindHashEntry(&tclSDDSCmdTable, argv[1]);
  if (entryPtr == NULL) {
    Tcl_SetResult(interp, "Unknown sdds command ", (Tcl_FreeProc *) NULL);
    return TCL_ERROR;
  } else {
    handler = (int(*)())Tcl_GetHashValue(entryPtr);
    return ((*handler)(interp, argc, argv));
  }

}

int TclSDDS_Init(Tcl_Interp *interp) {
  CMD_INFO *cmd;

  Tcl_CreateCommand(interp, "sdds", tclSDDSCmd, (ClientData *) NULL,
		    (Tcl_CmdDeleteProc *)NULL);

  Tcl_InitHashTable(&tclSDDSCmdTable, TCL_STRING_KEYS);
  for (cmd=tclSDDSCommands ; cmd->name != NULL ; cmd++) {
    int newEntry;
    Tcl_HashEntry *entryPtr;
    entryPtr = Tcl_CreateHashEntry(&tclSDDSCmdTable, cmd->name, &newEntry);
    Tcl_SetHashValue(entryPtr, cmd->proc);
  }
  tclSDDSInit(interp);
  
  return TCL_OK;
}
